* Spatial equilibrium lumber trade model
* Copyright G. Cornelis van Kooten

$Oneolcom
$eolcom #

SETS  s  supply regions /aus, bcc, bci, al, ac, roc, chl, chn, fin, jap, nz,
                 rus, swe, usn, uss, usw, rola, ROE, roa, row /
;
alias(s,d);

TABLE sup(s,*) supply info
      supply  sprice  selast
*   (000s m3)($/000m3)  elas of supply
aus   4602.7  226000     0.5
bcc     4603  204300   0.529
bci  16528.9  195100   0.529
al    4608.3  196100   0.415
ac    3227.6  214700    0.57
roc   9715.4  225400   0.415
chl   5862.7  211100     0.5
chn  25145.7  255000     0.5
fin     9008  227800     0.5
jap  15508.9  325000     0.5
nz    3946.6  212000     0.5
rus  23467.5  211100     0.5
swe  16173.2  248700     0.5
usn   4026.3  227200     0.2
uss  37391.7  193100   0.937
usw  16934.3  212500   0.866
rola 14429.2  205000       1
ROE  62519.5  275000       1
roa  15497.9  230000       1
row  23006.3  208900     1.5
;

TABLE dem(d,*) demand info
     demand   dprice   delast
*   (000s m3)($/000m3)  elas of demand
aus   5171.1  226000   -0.16
bcc   1837.4  204300   -0.16
bci    487.5  195100   -0.16
al    1923.3  196100   -0.16
ac    1198.4  214700   -0.16
roc  12087.3  225400   -0.16
chl   3777.7  211100   -0.21
chn  35406.6  255000   -0.21
fin   4345.1  227800   -0.16
jap  21763.7  325000   -0.16
nz    1720.5  212000   -0.16
rus  11310.2  211100   -0.14
swe   5633.8  248700   -0.16
usn  31142.2  227200   -0.16
uss  24578.2  193100   -0.16
usw  15963.9  212500   -0.16
rola 15308.2  205000   -0.56
ROE  77047.5  275000   -0.17
roa  16411.6  230000   -0.21
row  29089.8  208900    -0.2
;

TABLE lumflow(s,d)
$Include lumberflow.gms
;
TABLE SH(s,d)   Shipping & handling costs on lumber exported from s to d
$Include LumSH.gms
;
TABLE taxduty(s,d) Duties on lumber exported from s to d
$Include Duty.gms
;

* ------- Lumber Manufacuring Cost and Lumber Demand Parameters ------
PARAMETERS slope(s)     Slope parameter of lumber manufacturing cost
           sint(s)      Intercept parameter of manufacturing cost
           dslope(d)    Slope of lumber demand function
           dint(d)      Intercept of lumber demand function
           tax(s,d)     lumber tariffs imposed by demand region
;
* Assume elasticity of supply equals 1.0 so supply of lumber goes thru origin
slope(s) = sup(s,'sprice')/sup(s,'supply');
sint(s) = 0;
dslope(d) = -1*dem(d,'dprice')/(dem(d,'delast')*dem(d,'demand'));
dint(d) = dem(d,'dprice') + dslope(d)*dem(d,'demand');
tax(s,d) = dem(d,'dprice')*taxduty(d,s);  #Converts ad valorem tax to per unit

* ----------- CONSTRUCTION OF THE MODEL EQUATIONS
POSITIVE VARIABLES
     qd(d)      lumber consumption
     qs(s)      lumber supply
     x(s,d)     lumber trade from supply to demand regions
;

VARIABLE
    Z      total surplus in phase I problem
;

EQUATIONS
    obj     objective for PMP calibration phase

    ys(s)   lumber supply constraint
    yd(d)   lumber demand costraint
;

obj..  Z =E= sum(d, (dint(d)-.5*dslope(d)*qd(d))*qd(d))
              - sum(s, (sint(s)+.5*slope(s)*qs(s))*qs(s))
              - sum((s,d), SH(s,d)*x(s,d))   # mult by 1000 to get price
              - sum(s, sum(d, x(s,d)*tax(s,d))) ;

ys(s)..     sum(d, x(s,d)) =L= qs(s);
yd(d)..     qd(d) - sum(s, x(s,d)) =L= 0;

MODEL lumbertrade /obj, ys, yd/;

OPTION qcp = cplex;
SOLVE lumbertrade using QCP maximizing Z;
* ------------------------------------------------------

* ---------- Parameters to store results ---------------
PARAMETERS lumtax(s,d)    lumber taxes
           Trade(s,d)     Bilateral lumber trade
           Produced(s)    Total lumber production by region
           Consumed(d)    Total lumber consumption by region
           TotLumTax(s)   Total lumber tax paid by region $ mil
           pd(d)          Demand price of lumber $ per m3
           ps(s)          Supply price of lumber $ per m3
           CS(d)          'Consumer surplus ($ mil)'
           QR(s)          'Producer surplus or quasi rent ($ mil)'
;

   lumtax(s,d) = dem(d,'dprice')*taxduty(d,s);
   Trade(s,d) = round(x.l(s,d),2);
   Produced(s) = sum(d, x.l(s,d));
   Consumed(d) = sum(s, x.l(s,d));
*   pd(d) = dint(d) - dslope(d)*Cons(d);
   pd(d) = yd.m(d);
*   ps(s) = sint(s) + slope(s)*Total(s);
   ps(s) = ys.m(s);
   TotLumTax(s) = sum(d, x.l(s,d)*pd(d)*taxduty(s,d))/1000000;
   CS(d) = (.5*(dint(d)-pd(d))*qd.l(d))/1000000;  # Consumer surplus
   QR(s) = (.5*ps(s)*qs.l(s))/1000000;  # Quasi rent

OPTION Trade:1:1:1; option pd:2:0:10; option pd:2:0:10;
option TotLumTax:1:0:1; option CS:1:0:10; option QR:1:0:10;

DISPLAY Trade, pd, ps, TotLumTax, CS, QR;

$call gdxxrw.exe TradeResult.xls o=ExportData.gdx SQ=N @export.txt

execute_unload "TradeResult.gdx" pd, ps
execute 'gdxxrw.exe TradeResult.gdx O=TradeResult.xls SQ=N text="Lumber Demand Prices" rng=Price!a1 par=pd rng=Price!a2 rdim=1 text="Lumber Supply Prices" rng=Price!c1 par=ps rng=Price!c2 rdim=1'

execute_unload "TradeResult.gdx" Produced, Consumed
execute 'gdxxrw.exe TradeResult.gdx O=TradeResult.xls SQ=N text="Lumber Production" rng=Production!a1 par=Produced rng=Production!a2 rdim=1 text="Lumber Consumption" rng=Production!a23 par=Consumed rng=Production!a24 rdim=1'

execute_unload "TradeResult.gdx" CS, QR
execute 'gdxxrw.exe TradeResult.gdx Output=TradeResult.xls SQ=N text="Consumer Surplus ($mil)" rng=Welfare!b1 par=CS rng=Welfare!a2 rdim=1 text="Producer Surplus ($mil)" rng=Welfare!d1 par=QR rng=Welfare!c2 rdim=1'

execute_unload "TradeResult.gdx" Trade
execute 'gdxxrw.exe TradeResult.gdx Output=TradeResult.xls SQ=N text="Bilateral trade (00s m3)" rng=Tradeflow!a1 par=Trade rng=Tradeflow!a2 rdim=1 cdim=1'
